### R code from vignette source 'pulicaria.Rnw'
### Encoding: UTF-8

###################################################
### code chunk number 1: pulicaria.Rnw:57-71
###################################################
#require some packages needed later
data <- read.csv('data.csv')   #Load the  data
data <- data[data$Days!=-9999,]      #Drop unobserved extinctions

#Define colors for plots
ose1 <- rgb(85,108,17, m=255)
ose2 <- rgb(160,108,17, m=255)
ose3 <- rgb(114,132,56, m=255)
ose4 <- rgb(137,152,87, m=255)

#Obtain approximate number of reproductive events assuming 75 inidividuals per population and reproducing 10 offspring every 8 days
repro <- 75*10*sum(data$Days)/8
#Obtain approximate number of beneficial mutations
mutations <- 4e-9*repro


###################################################
### code chunk number 2: histogram
###################################################
pdf(file='pulicaria-histogram.pdf', w=6.5, h=4); par(mfrow=c(1,2))
hist(data$Days, col=ose1, main='', xlab='Extinction time (days)')
hist(log(data$Days), col=ose1, main='', xlab='Extinction time (log days)')
graphics.off()


###################################################
### code chunk number 3: multihistogram
###################################################
blocks <- unique(data$Block)
#x11(w=12,h=8)
#pdf('pulicaria-multihistogram.pdf', w=12, h=8)
par(mfrow=c(3,7), mar=c(3,1,1,1), oma=c(5,5,1,1))
for(b in blocks){
  hist(data$Days[data$Block==b],
  #xlim=c(0,1400),
  main=paste('Block ', b),
  col=ose1)
}
mtext(side=1, line=5, 'Extinction time (days)                                                                                                                                  ', cex=1.6)
mtext(side=2, line=76, '                                                                                 Frequency', cex=1.4)
#dev.off()


###################################################
### code chunk number 4: ecdf
###################################################
blocks <- unique(data$Block)
#x11(w=12,h=8)
#pdf('pulicaria-multihistogram.pdf', w=12, h=8)
par(mfrow=c(3,7), mar=c(3,1,2,1), oma=c(5,5,1,1))
for(b in blocks){
  e0 <- ecdf(data$Days[data$Block==b])
  plot(e0, main =paste('Block ', b) )
}
mtext(side=1, line=5, 'Extinction time (days)                                                                                                                                  ', cex=1.6)
mtext(side=2, line=76, '                                                                               Cumulative probability', cex=1.4)
#dev.off()


###################################################
### code chunk number 5: statistics
###################################################
require(moments)
block.statistics <- data.frame(Block=seq(1,21), mean=numeric(21), median=numeric(21), excess.kurtosis=numeric(21), skewness=numeric(21), variance=numeric(21), cv=numeric(21))
block.statistics$mean <- aggregate(data$Days, by=list(data$Block), mean)[,2]                    #mean
block.statistics$median <- aggregate(data$Days, by=list(data$Block), median)[,2]                #median
block.statistics$excess.kurtosis <- aggregate(data$Days, by=list(data$Block), kurtosis)[,2] - 3 #kurtosis
block.statistics$skewness <- aggregate(data$Days, by=list(data$Block), skewness)[,2]            #skewness
block.statistics$variance <- aggregate(data$Days, by=list(data$Block), var)[,2]                 #variance
block.statistics$cv <- sqrt(block.statistics$variance)/block.statistics$mean                         #coefficient of variation

# par(mar=c(3,4,2,2)+0.1)
# layout(matrix(c(1,1,1,2,3,3,3,4,5,5,5,6,7,7,7,8,9,9,9,9,9,9,9,9), 6, 4, byrow = TRUE))
# plot(block.statistics$mean, xlab='', ylab='Mean', xaxp=c(1,21,20), cex.axis=0.7, col=ose1, main='')
# hist(block.statistics$mean, main='', xlab='', cex.axis=0.7, col=ose1)
# plot(block.statistics$median, xlab='', ylab='Median', xaxp=c(1,21,20), cex.axis=0.7, col=ose1)
# hist(block.statistics$median, main='', xlab='', cex.axis=0.7, col=ose1)
# plot(block.statistics$excess.kurtosis, xlab='', ylab='Excess kurtosis', xaxp=c(1,21,20), cex.axis=0.7, col=ose1)
# hist(block.statistics$excess.kurtosis, main='', xlab='', cex.axis=0.7, col=ose1)
# plot(block.statistics$skewness, xlab='', ylab='Skewness', xaxp=c(1,21,20), cex.axis=0.7, col=ose1)
# hist(block.statistics$skewness, main='', xlab='', cex.axis=0.7, col=ose1)
# boxplot(Days~Block, data=data, col=ose1, xlab='Block', ylab='Extinction time (days)', cex.axis=0.6)
# mtext(side=3, line=1.7, 'Block                                                         ', cex=0.7)
# mtext(side=1, line=1.2, 'Block                                                                                                                                                                                            ', cex=0.7)


par(mar=c(4,4,0,2)+0.1)
layout(matrix(c(1,1,1,2,3,3,3,4,5,5,5,6,7,7,7,8), 4, 4, byrow = TRUE))
plot(block.statistics$mean, xlab='', ylab='Mean', xaxp=c(1,21,20), cex.axis=0.7, col=ose1, main='')
hist(block.statistics$mean, main='', xlab='', cex.axis=0.7, col=ose1)
plot(block.statistics$median, xlab='', ylab='Median', xaxp=c(1,21,20), cex.axis=0.7, col=ose1)
hist(block.statistics$median, main='', xlab='', cex.axis=0.7, col=ose1)
plot(block.statistics$excess.kurtosis, xlab='', ylab='Excess kurtosis', xaxp=c(1,21,20), cex.axis=0.7, col=ose1)
hist(block.statistics$excess.kurtosis, main='', xlab='', cex.axis=0.7, col=ose1)
plot(block.statistics$skewness, xlab='', ylab='Skewness', xaxp=c(1,21,20), cex.axis=0.7, col=ose1)
mtext(side=1, line=3, 'Block')

hist(block.statistics$skewness, main='', xlab='', cex.axis=0.7, col=ose1)
#boxplot(Days~Block, data=data, col=ose1, xlab='Block', ylab='Extinction time (days)', cex.axis=0.6)
#mtext(side=3, line=1.7, 'Block                                                         ', cex=0.7)
#mtext(side=1, line=1.2, 'Block                                                                                                                                                                                            ', cex=0.7)

names(block.statistics) <- c('Block', 'Mean', 'Median', 'Excess kurtosis', 'Skewness', 'Variance', 'Coefficient of variation')


###################################################
### code chunk number 6: table
###################################################
require(xtable)
print(xtable(block.statistics, print.results=TRUE, align=c('c', 'c','c','c','c','c','c','c'), caption='Summary statistics of extinction time by block. Boxplots show interquartile range; whiskers extend to the most extreme points within a distance of 1.5 times the interquartile range. Note the logarithmic axis on extinction time plot.'),include.rownames=FALSE)


###################################################
### code chunk number 7: pulicaria.Rnw:198-199
###################################################
require(MIfuns)


###################################################
### code chunk number 8: pulicaria.Rnw:202-204
###################################################
T <- ftable(xtabs(~Block+Tetracycline+Startno,data=data), row.vars=1)
print(xtable(ftable2data.frame(T),align=c('c','c','c','c','c','c','c'), caption='Number of experimental populations by block and treatment.'), include.rownames=FALSE)


###################################################
### code chunk number 9: model1
###################################################
require(lme4)
Model1 <- lmer(log(Days) ~ Tetracycline + Startno + (1|Block), data=data, REML=FALSE)
#summary(Model1)


###################################################
### code chunk number 10: model2
###################################################
Model2 <- lmer(log(Days) ~ Tetracycline + (1|Block), data=data, REML=FALSE)
#summary(Model2)


###################################################
### code chunk number 11: model3
###################################################
Model3 <- lmer(log(Days) ~ Startno + (1|Block), data=data, REML=FALSE)
#summary(Model3)


###################################################
### code chunk number 12: model4
###################################################
Model4 <- lmer(log(Days) ~ (1|Block), data=data, REML=FALSE)
#summary(Model4)


###################################################
### code chunk number 13: aov
###################################################
model.aov <- aov(log(Days) ~ Tetracycline + Startno, data=data)  #fixed effects


###################################################
### code chunk number 14: mcmc
###################################################
samp1 <- mcmcsamp(Model1, n=1e5)  #set to 1e5 for final
CI <- HPDinterval(samp1)$fixef


###################################################
### code chunk number 15: anova
###################################################
print(xtable(anova(Model1, Model2, Model3, Model4), caption='ANOVA table for estimated treatment effects in four linear mixed effects models'), include.rownames=TRUE)


###################################################
### code chunk number 16: diagnostics
###################################################
par(mfrow=c(2,2))
plot(fitted(Model1), log(data$Days), xlab='Fitted value', ylab='Observed value' )
abline(b=1,a=0,col=ose1, lty=2, lwd=2)
plot(fitted(Model1), residuals(Model1), xlab='Fitted value', ylab='Residuals', col=ifelse(data$Startno==3,ose1,ose2), pch=ifelse(data$Tetracycline==0,1,2))
box()
boxplot(residuals(Model1)~data$Tetracycline, ylab='Residuals', xaxt='n', xlab='Tetracycline')
axis(1, at=c(1,2), labels=c('No', 'Yes'))
box()
boxplot(residuals(Model1)~data$Startno, border=c(ose1, ose2), ylab='Residuals', xlab='Initial populations size')
box()


###################################################
### code chunk number 17: pulicaria.Rnw:303-312
###################################################
#Load required libraries
source('Clauset-power-law-code/zeta.R')
source('Clauset-power-law-code/discpowerexp.R')
source('Clauset-power-law-code/discweib.R')
source('Clauset-power-law-code/weibull.R')
#source('Clauset-power-law-code/disclnorm.R')
source('Clauset-power-law-code/discexp.R')
#source('Clauset-power-law-code/power-law-test.R') #likelihood testing of power law vs alternatives
source('Clauset-power-law-code/plfit.r')


###################################################
### code chunk number 18: compare
###################################################
discweib.fit.jmd <- function(x,threshold=0) {
  #This version is a modification of discweib.fit that differs from Clauset et al. simply by passing parameters as logarithm of the usual definition
  #i.e., anti-logged when calculating log-likelihood and logged when returning the estimate
  # Start off with a rough-and-ready estimator for continuous data
  theta_0 <- weibull.est.shape.inefficient(x)
  # Trim the data set
  x <- x[x>=threshold]
  n <- length(x)
  # define the likelihood
  negloglike <- function(theta) {
    -discweib.loglike(x,exp(theta[1]),exp(theta[2]),threshold)
  }
  # optimize
  est <- nlm(f=negloglike,p=c(log(theta_0[1]),log(theta_0[2])))
  fit <- list(type="discweib", shape=exp(est$estimate[1]), scale=exp(est$estimate[2]),
              loglike =-est$minimum, threshold=threshold,
              samples.over.threshold=n)
  return(fit)
}

data.summary.compare <- data.frame(blocks=seq(1:21), weib.ll.jmd = rep(NaN,21),  weib.ll.clauset = rep(NaN,21))
for(i in c(1:7, 9:13, 15:21)){    #eventually i in 1:21
  data.0 <- data[data$Block==blocks[i],]
  threshold <- plfit(data.0$Days,"limit", nosmall=FALSE, finite=TRUE)$xmin
  data.summary.compare$weib.ll.jmd[i] <- discweib.fit.jmd(data.0$Days, threshold=threshold)$loglike
  data.summary.compare$weib.ll.clauset[i] <- discweib.fit(data.0$Days, threshold=threshold)$loglike
}
pdf(file='compare-likelihoods.pdf', w=4, h=4)
plot(data.summary.compare$weib.ll.jmd - data.summary.compare$weib.ll.clauset, ylab='log-likelihood difference', xlab='Block', cex.lab=0.7, axes=FALSE)
axis(1,at=seq(1:21), cex.axis=0.6)
axis(2, cex.axis=0.6)
graphics.off()


###################################################
### code chunk number 19: fitTails
###################################################
data.summary <- data.frame(Block=seq(1:21))
#pdf(file='histograms.pdf', w=12, h=8); par(mfrow=c(3,7))

#code commented because the other piece has shape parameters

for(i in 1:21){
  data.0 <- data[data$Block==blocks[i],]
  data.summary$Threshold[i] <- threshold <- plfit(data.0$Days,"limit", nosmall=FALSE, finite=TRUE)$xmin
  data.tail <- data.0$Days[data.0$Days>=threshold] # records in the tail
#  hist(data.tail-threshold, col=ose3, main=paste('Block', i), xlab='Extinction time (days)')
#  hist(data.0$Days, col=ose3, main=paste('Block', i), xlab='Extinction time (days)')
  data.summary$n.tail[i] <- length(data.tail)    #count of observations over threshold
#  data.summary$mean[i] <- mean(data.0$Days)
#  data.summary$median[i] <- median(data.0$Days)
#  data.summary$skewness[i] <- skewness(data.0$Days)
#  data.summary$var[i] <- var(data.0$Days)
#  data.summary$cv[i] <- sd(data.0$Days)/mean(data.0$Days)
  
  data.summary$power.ll[i] <- zeta.fit(data.0$Days, threshold=threshold)$loglike
  data.summary$power.exp.ll[i] <- discpowerexp.fit(data.0$Days, threshold=threshold)$loglike
  data.summary$weib.ll[i] <- discweib.fit.jmd(data.0$Days, threshold=threshold)$loglike
  data.summary$exp.ll[i] <- discexp.fit(data.0$Days, threshold=threshold)$loglike
}


# for(i in 1:21){    #eventually i in 1:21
#   data.0 <- data[data$Block==blocks[i],]
#   threshold <- quantile(data.0$Days, prob=0.5)
#   data.summary$n[i] <- dim(data.0)[1]
#   data.summary$mean[i] <- mean(data.0$Days)
#   data.summary$median[i] <- median(data.0$Days)
#   data.summary$skewness[i] <- skewness(data.0$Days)
#   data.summary$var[i] <- var(data.0$Days)
#   data.summary$cv[i] <- sd(data.0$Days)/mean(data.0$Days)
#   data.summary$power.ll[i] <- zeta.fit(data.0$Days, threshold=threshold)$loglike
#   data.summary$power.exp.ll[i] <- discpowerexp.fit(data.0$Days, threshold=threshold)$loglike
#   #data.summary$mass.lnorm.ll[i] <- fitdistr(data.0$Days[data.0$Days>threshold]-threshold,"lognormal")$loglik
#   tmp <- discweib.fit(data.frame(data.0$Days), threshold=threshold)
#   data.summary$weibull.ll[i] <- tmp$loglik
#   #data.summary$lnorm.ll[i] <- fit.lnorm.disc(data.frame(data.0$Days), threshold=threshold)$loglik
#   data.summary$exp.ll[i] <- discexp.fit(data.0$Days, threshold=threshold)$loglike
#   data.summary$weibull.shape.parm[i] <- tmp$shape
# }

#graphics.off()

#calculate AICc values (AICc = - 2ll + 2k + 2k(k+1)/(n-k-1))
data.summary$power.AICc <- -2*(data.summary$power.ll) + 2*1 + 2*1*(1+1)/(data.summary$n.tail-1-1)
data.summary$power.exp.AICc <- -2*(data.summary$power.exp.ll) + 2*2 + 2*2*(2+1)/(data.summary$n.tail-2-1)
data.summary$weib.AICc <- -2*(data.summary$weib.ll) + 2*2 + 2*2*(2+1)/(data.summary$n.tail-2-1)
data.summary$exp.AICc <- -2*(data.summary$exp.ll) + 2*1 + 2*1*(1+1)/(data.summary$n.tail-1-1)

data.to.print <- data.summary[,c(1:3, 8:11)]
colnames(data.to.print) <- c('Block', 'Threshold', 'n', 'Power','Power/cutoff', 'Weibull', 'Exponential')


###################################################
### code chunk number 20: pulicaria.Rnw:458-459
###################################################
print(xtable(data.frame(AIC=colSums(data.summary)[8:11], row.names=c('Power','Power cutoff','Weibull','Exponential')), caption='Total statistical support (summed AIC) for four models of tail extinction probabilities.'), row.numbers=FALSE)


###################################################
### code chunk number 21: pulicaria.Rnw:468-494
###################################################
BlockRx <- unique(paste(data$Block, data$Startno))
data.summary.2 <- data.frame(id=seq(1:42))

for(i in 1:42){
  data.0 <- data[paste(data$Block, data$Startno)==BlockRx[i],]
  data.summary.2$threshold[i] <- threshold <- plfit(data.0$Days,"limit", nosmall=FALSE, finite=TRUE)$xmin
  data.summary.2$id[i] <- BlockRx[i]
  data.summary.2$Startno[i] <- data.0$Startno[1]
  data.summary.2$n.tail[i] <- length(data.0$Days[data.0$Days>=threshold])
  data.summary.2$power.ll[i] <- zeta.fit(data.0$Days, threshold=threshold)$loglike
  data.summary.2$power.exp.ll[i] <- discpowerexp.fit(data.0$Days, threshold=threshold)$loglike
  data.summary.2$weib.ll[i] <- discweib.fit.jmd(data.0$Days, threshold=threshold)$loglike
  data.summary.2$exp.ll[i] <- discexp.fit(data.0$Days, threshold=threshold)$loglike
}


#calculate AICc values (AICc = - 2ll + 2k + 2k(k+1)/(n-k-1))
data.summary.2$power.AICc <- -2*(data.summary.2$power.ll) + 2*1 + 2*1*(1+1)/(data.summary.2$n.tail-1-1)
data.summary.2$power.exp.AICc <- -2*(data.summary.2$power.exp.ll) + 2*2 + 2*2*(2+1)/(data.summary.2$n.tail-2-1)
data.summary.2$weib.AICc <- -2*(data.summary.2$weib.ll) + 2*2 + 2*2*(2+1)/(data.summary.2$n.tail-2-1)
data.summary.2$exp.AICc <- -2*(data.summary.2$exp.ll) + 2*1 + 2*1*(1+1)/(data.summary.2$n.tail-1-1)

data.to.print2 <- data.summary.2[,c(1:4,9:12)]
colnames(data.to.print2) <- c('Block/x(0)', 'Threshold', 'x(0)', 'n', 'Power', 'Power/cutoff', 'Weibull', 'Exponential')

print(xtable(data.to.print2, caption='Statistical fit of tail extinction times, by block-treatment combinations, to four models.'),include.rownames=FALSE)


###################################################
### code chunk number 22: pulicaria.Rnw:499-510
###################################################
N03 <- data.summary.2[data.summary.2$Startno==3,9:12]
N018 <- data.summary.2[data.summary.2$Startno==18,9:12]

aic.by.Startno <-t(data.frame(
  x3=colSums(N03[c(1:12,14:21),]),
  x18=colSums(N018[1:21,])))

rownames(aic.by.Startno) <- c('x(0)=3','x(0)=18')
colnames(aic.by.Startno) <- c('Power','Power/cutoff','Weibull','Exponential')

print(xtable(aic.by.Startno, caption='Total statistical support (summed AIC) for four models of tail extinction probabilities for two initial population sizes.'), row.numbers=FALSE)


